#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/colours.rb'
# =========================================================================== #
require 'environment_information/colours/colours.rb'

class EnvironmentInformation::EnvironmentInformation

  # ========================================================================= #
  # === TRY_TO_USE_HTML_COLOURS
  #
  # If true then we will try to use the konsole-colours, when possible.
  #
  # These are part of the colours gem, so this adds a dependency on the
  # colours gem.
  # ========================================================================= #
  TRY_TO_USE_HTML_COLOURS = true

  # ========================================================================= #
  # === colourize_this_in_the_right_side_colour
  # ========================================================================= #
  def colourize_this_in_the_right_side_colour(i)
    if use_colours?
      ::Colours.send(::EnvironmentInformation.colour_for_the_right_side, i)
    else
      i
    end
  end

  # ========================================================================= #
  # === set_use_colours
  # ========================================================================= #
  def set_use_colours(i = '')
    ::EnvironmentInformation.set_use_colours(true)
  end

  # ========================================================================= #
  # === use_colours?
  # ========================================================================= #
  def use_colours?
    ::EnvironmentInformation.use_colours?
  end

  # ========================================================================= #
  # === enable_colours
  # ========================================================================= #
  def enable_colours
    ::EnvironmentInformation.use_colours = true
  end

  # ========================================================================= #
  # === disable_colours
  #
  # Call this method when you wish to disable the colours. Note that
  # this will disable colours on the toplevel.
  # ========================================================================= #
  def disable_colours
    ::EnvironmentInformation.use_colours = false
  end; alias no_colours disable_colours # === no_colours()

  # ========================================================================= #
  # === rev
  # ========================================================================= #
  def rev
    if use_colours?
      Colours::GREEN
    else
      ''
    end
  end

  # ========================================================================= #
  # === true_rev
  # ========================================================================= #
  def true_rev
    if use_colours?
      Colours::REV
    else
      ''
    end
  end

  # ========================================================================= #
  # === col1
  #
  # The "first" colour to be used.
  # ========================================================================= #
  def col1
    if use_colours?
      Colours::BOLD_BLUE
    else
      ''
    end
  end

  # ========================================================================= #
  # === cyan
  # ========================================================================= #
  def cyan(i = '')
    if i.empty?
      Colours::CYAN
    end if use_colours?
  end

  # ========================================================================= #
  # === blue
  # ========================================================================= #
  def blue(i = '')
    if i.empty?
      Colours::BLUE
    end if use_colours?
  end

  # ========================================================================= #
  # === simp
  # ========================================================================= #
  def simp(i)
    return ::Colours.simp(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === sfile
  # ========================================================================= #
  def sfile(i)
    return ::Colours.sfile(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === sfancy
  # ========================================================================= #
  def sfancy(i = '')
    return ::Colours.sfancy(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === crimson
  # ========================================================================= #
  def crimson(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.crimson(i)
    end
    return i
  end

  # ========================================================================= #
  # === darkgreen
  # ========================================================================= #
  def darkgreen(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.darkgreen(i)
    end
    return i
  end

  # ========================================================================= #
  # === dimgray
  # ========================================================================= #
  def dimgray(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.dimgray(i)
    end
    return i
  end

  # ========================================================================= #
  # === royalblue
  # ========================================================================= #
  def royalblue(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.royalblue(i)
    end
    return i
  end

  # ========================================================================= #
  # === slateblue
  # ========================================================================= #
  def slateblue(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.slateblue(i)
    end
    return i
  end

  # ========================================================================= #
  # === steelblue
  # ========================================================================= #
  def steelblue(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.steelblue(i)
    end
    return i
  end

  # ========================================================================= #
  # === seagreen
  # ========================================================================= #
  def seagreen(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.seagreen(i)
    end
    return i
  end

  # ========================================================================= #
  # === olivedrab
  #
  # This one will be used for the right-hand side.
  # ========================================================================= #
  def olivedrab(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.olivedrab(i)
    end
    return i
  end

  # ========================================================================= #
  # === olive
  # ========================================================================= #
  def olive(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.olive(i)
    end
    return i
  end

  # ========================================================================= #
  # === limegreen
  # ========================================================================= #
  def limegreen(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.limegreen(i)
    end
    return i
  end

  # ========================================================================= #
  # === darkolivegreen
  # ========================================================================= #
  def darkolivegreen(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.darkolivegreen(i)
    end
    return i
  end

  # ========================================================================= #
  # === lightgreen
  # ========================================================================= #
  def lightgreen(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.lightgreen(i)
    end
    return i
  end

  # ========================================================================= #
  # === mediumpurple
  # ========================================================================= #
  def mediumpurple(i = '')
    if use_colours? and TRY_TO_USE_HTML_COLOURS
      i = ::Colours.mediumpurple(i)
    end
    return i
  end

  # ========================================================================= #
  # === teal
  # ========================================================================= #
  def teal(i = '')
    return ::Colours.teal(i) if use_colours?
    return i
  end

  # ========================================================================= #
  # === orange
  # ========================================================================= #
  def orange(i = '')
    return ::Colours.orange(i) if use_colours?
    return i
  end

end