#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/help.rb'
# =========================================================================== #
require 'environment_information/requires/require_asciitable.rb'

module EnvironmentInformation

class EnvironmentInformation

  # ========================================================================= #
  # === do_show_help                                               (help tag)
  #
  # Show the commandline-usage through this method here.
  #
  # To invoke this, try:
  #
  #   envi --help
  #
  # ========================================================================= #
  def do_show_help
    e
    lpad = "#{col1}    "
    e "#{rev}The following options are available for "\
      "#{simp('class EnvironmentInformation:')}#{rev}#{N}#{N}"
    if ASCIITABLE_IS_AVAILABLE
      e col1+'    asciitable        '+rev+'# Print in Ascii Table format'
    end
    e lpad+'html              '+rev+'# Save the environment information '\
      'into a .html file'
    e lpad+'gui               '+rev+'# Start the GTK gui bindings; some '\
      'aliases are possible such as --GUI'
    e lpad+'nocolours         '+rev+'# Disable colours '\
      '(--disable-colours also works)'
    e lpad+'noruby            '+rev+'# dont show ruby-related environment '\
      'information'
    e lpad+'help              '+rev+'# show some help'
    e lpad+'xorg              '+rev+'# show some xorg-specific '\
      'components (also --xorg-components)'
    e lpad+'OS                '+rev+'# show the OS then exit'
    e lpad+'full              '+rev+'# show full information, including '\
      'GTK, Glib, Atk and Pango Version'
    e '                      # ^^^ This is probably the most useful usage.'
    e lpad+'openssl           '+rev+'# display openssl option'
    e lpad+'REALLY_ALL        '+rev+'# also compare the program_versions '\
      '(--really-everything is an alias to this)'
    e lpad+'save              '+rev+'# display the result, then '\
      'save it into a file'
    e lpad+'version           '+rev+'# show the version of '\
      'EnvironmentInformation in use'
    e lpad+'--show-remote-url '+rev+'# show the remote URLs '\
      'of a program (requires the RBT project)'
    e lpad+'padding=value     '+rev+'# set to another padding value'
    e rev+N+'The above commands should be commandline arguments, '\
      'such as: "'+teal('envi full')+rev+'".'
    e
    e rev+'Do note that you can prefix the above commands via '\
      'a leading --, of course.'
    e
    # Show a simple usage example next:
    e 'Usage example:'
    e
    e steelblue(
        '    envi --version'
      )
    e
    e rev+'If you wish to show only some programs, you could '\
      'use the following:'
    e
    e steelblue(
        '    envi --use-these-programs=bash,binutils,bison,yacc,bzip2,coreutils,diff,find,gawk,gcc'
      )
    e steelblue(
        '    envi --use-these-programs=:lfs'
      )
    e true_rev # Reset via Colours.rev here.
    do_exit_the_program
  end; alias show_help do_show_help # === show_help

end; end