#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/html.rb'
# =========================================================================== #
require 'environment_information/toplevel_methods/write_what_into.rb'
require 'environment_information/constants/temp_directory.rb'

module EnvironmentInformation

class EnvironmentInformation 

  # ========================================================================= #
  # === do_generate_a_html_file                                    (html tag)
  #
  # The action-method that will generate the html file. This depends
  # on the gem html_tags, with module HtmlTags though.
  #
  # To invoke this method, do:
  #
  #   envi --generate-html-file
  #
  # ========================================================================= #
  def do_generate_a_html_file(
      styling_instructions = :bold
    )
    # ======================================================================= #
    # === Ensure that the html_tags project is available
    # ======================================================================= #
    unless Object.const_defined? :HtmlTags
      begin
        require 'html_tags'
      rescue LoadError
        e 'html_tags is not available. Considering installing it via:'
        e
        e '  gem install html_tags'
        e
      end
    end
    # ======================================================================= #
    # === Define where to store the .html file in question
    # ======================================================================= #
    this_html_file = "#{::EnvironmentInformation.temp_directory?}environment_information.html"
    what = dataset_as_string
    case styling_instructions
    when :bold
      what = HtmlTags.b(what)
    end
    what = HtmlTags.html(
      HtmlTags.body(
        HtmlTags.h1(
          'Your environment', css_style: 'margin-top:1em'
        )+
        HtmlTags.pre(what, css_style: 'font-size: larger; padding-left:1em;')
      )
    )
    ::EnvironmentInformation.write_what_into(what, this_html_file)
    # ======================================================================= #
    # Notify the user what has been done.
    # ======================================================================= #
    if File.exist? this_html_file
      opnn; e "Generated a HTML file at `#{sfile(this_html_file)}`."
    end
  end

end; end