#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/misc.rb'
# =========================================================================== #
require 'environment_information/toplevel_methods/cd.rb'
require 'environment_information/toplevel_methods/misc.rb'
require 'environment_information/toplevel_methods/write_what_into.rb'
require 'environment_information/version/version.rb'
require 'environment_information/constants/misc.rb'
require 'environment_information/constants/array_tracked_components.rb'

module EnvironmentInformation

class EnvironmentInformation

  begin
    require 'rbt/utility_scripts/report_mate_desktop_version.rb'
  rescue LoadError; end

  # ========================================================================= #
  # === return_remote_gtk2_version
  # ========================================================================= #
  def return_remote_gtk2_version
    ::EnvironmentInformation.return_remote_gtk2_version
  end

  # ========================================================================= #
  # === work_on_the_programs_directory_only
  #
  # This works on the /home/Programs/ directory directly.
  #
  # Invocation example:
  #
  #   envi --work-on-programs-directory-only
  #
  # ========================================================================= #
  def work_on_the_programs_directory_only(
      use_this_as_programs_directory = '/home/Programs/'
    )
    # ======================================================================= #
    # 1) First, we have to determine which programs are available.
    # ======================================================================= #
    e "#{rev}Determining which programs are available at the prefix "\
      "#{steelblue(use_this_as_programs_directory)}:"
    array_these_programs_are_available = []
    ARRAY_TRACKED_PROGRAMS.each {|entry|
      target = "#{use_this_as_programs_directory}#{entry.capitalize}"
      if File.directory?(target)
        # =================================================================== #
        # In this case we know that this target exists.
        # =================================================================== #
        array_these_programs_are_available << entry
      end
    }
    # ======================================================================= #
    # 2) Checking these programs next.
    # ======================================================================= #
    clear_main_dataset
    add(array_these_programs_are_available)
    ::EnvironmentInformation.set_prefix_to_use(use_this_as_programs_directory)
  end

  # ========================================================================= #
  # === set_main_array
  # ========================================================================= #
  def set_main_array(i)
    @array_display_these_components = i
  end; alias set_display_these_components               set_main_array # === set_display_these_components
       alias set_use_this_as_main_input                 set_main_array # === set_use_this_as_main_input
       alias show_only_the_components_from_this_dataset set_main_array # === show_only_the_components_from_this_dataset

  # ========================================================================= #
  # === clear_old_dataset
  # ========================================================================= #
  def clear_old_dataset
    @array_display_these_components = []
  end

  # ========================================================================= #
  # === clear_the_main_dataset
  # ========================================================================= #
  def clear_the_main_dataset
    clear_old_dataset
    do_not_display_the_ruby_components
  end; alias clear_main_dataset clear_the_main_dataset # === clear_main_dataset
       alias empty_main_dataset clear_the_main_dataset # === empty_main_dataset
       alias empty_main_array   clear_the_main_dataset # === empty_main_array

  # ========================================================================= #
  # === shall_we_really_store_which_programs_are_not_up_to_date?
  #
  # This method will also honour whether the user is on a roebe-like
  # system or whether the user is not.
  # ========================================================================= #
  def shall_we_really_store_which_programs_are_not_up_to_date?
    is_on_roebe? and !@array_these_programs_not_up_to_date.empty?
  end

  # ========================================================================= #
  # === consider_storing_which_programs_are_not_up_to_date
  # ========================================================================= #
  def consider_storing_which_programs_are_not_up_to_date
    if shall_we_really_store_which_programs_are_not_up_to_date?
      into = FILE_THESE_PROGRAMS_CAN_BE_UPGRADED
      what = YAML.dump(@array_these_programs_not_up_to_date)
      if File.directory?('/home/Temp/rbt/')
        into = "/home/Temp/rbt/#{File.basename(into)}"
      else
        into = "#{log_dir?}#{File.basename(into)}"
      end
      opnn; e 'We will also store which programs are not up to date.'
      opnn; e "These will be stored into the file at `#{sfile(into)}`."
      ::EnvironmentInformation.write_what_into(what, into)
    end
  end

  # ========================================================================= #
  # === result_as_array
  #
  # The lines that we will have inside of this method, may look like this:
  #
  #   "  operating_system:                 GNU/Linux\n"
  #
  # ========================================================================= #
  def result_as_array
    _ = dataset_as_string
    splitted = _.split("\n")
    splitted.map! {|inner_line|
      inner_line.split(': ').map {|entry| entry.strip }
    }
    return splitted
  end; alias result result_as_array # === result

  # ========================================================================= #
  # === load_dataset_from_this_file
  #
  # This method can be used to read which programs will be displayed
  # from a local file, rather than rely on the pre-set default.
  # ========================================================================= #
  def load_dataset_from_this_file(i)
    if File.exist? i
      File.readlines(i).map {|entry|
        entry.delete('-').strip # <- Clean up the input a little bit.
      }
    else
      opnn; e "No file exists at `#{sfile(i)}`."
    end
  end; alias try_to_load_dataset_from_this_file load_dataset_from_this_file # === try_to_load_dataset_from_this_file
       alias try_to_read_content_from_this_file load_dataset_from_this_file # === try_to_read_content_from_this_file

  # ========================================================================= #
  # === set_commandline_arguments
  # ========================================================================= #
  def set_commandline_arguments(i = '')
    i = [i].flatten.compact
    @commandline_arguments = i
  end

  # ========================================================================= #
  # === commandline_arguments?
  # ========================================================================= #
  def commandline_arguments?
    @commandline_arguments
  end

  # ========================================================================= #
  # === first_argument?
  # ========================================================================= #
  def first_argument?
    @commandline_arguments.first
  end

  # ========================================================================= #
  # === register_unavailable_program
  #
  # Programs that were not be found can be registered through this method.
  # ========================================================================= #
  def register_unavailable_program(i)
    i = i.to_sym # Let's store only Symbols.
    @array_unavailable_programs << i
  end; alias register_not_found register_unavailable_program # === register_not_found

  # ========================================================================= #
  # === use_ascii_table
  #
  # We will display in ascii-table format here.
  # ========================================================================= #
  def use_ascii_table
    disable_colours # Can't use colours right now. Perhaps at a later time.
    extend Terminal::Table::TableHelper
    @table = table ['Name', 'Version']
    @table.style = {
      padding_left:  2,
      width:       110 # Set the width here.
    }
    @use_ascii_table = true
  end

  # ========================================================================= #
  # === show_everything?
  # ========================================================================= #
  def show_everything?
    @show_everything
  end

  # ========================================================================= #
  # === open_this_file_in_editor
  # ========================================================================= #
  def open_this_file_in_editor
    _ = "bluefish #{THIS_FILE}"
    e _; system _; exit
  end

  # ========================================================================= #
  # === log_dir?
  # ========================================================================= #
  def log_dir?
    ::EnvironmentInformation.temp_directory?
  end; alias log_directory? log_dir? # === log_directory?

  # ========================================================================= #
  # === do_not_save_anything
  # ========================================================================= #
  def do_not_save_anything
    @shall_the_results_be_saved = false
    # ======================================================================= #
    # We will also avoid saving the results into a local yaml file.
    # ======================================================================= #
    @store_the_results_into_local_files = false
  end

  # ========================================================================= #
  # === consider_storing_the_components_that_were_displayed
  # ========================================================================= #
  def consider_storing_the_components_that_were_displayed
    ::EnvironmentInformation.consider_storing_these_results_into_a_local_file
  end; alias consider_storing_which_components_were_displayed consider_storing_the_components_that_were_displayed # === consider_storing_which_components_were_displayed

  # ========================================================================= #
  # === do_show_everything                         (everything tag, full tag)
  #
  # This method can be used when the user wishes to enable seeing
  # full information about his local environment (on the computer).
  #
  # Commandline invocation:
  #
  #   envi --everything
  #
  # ========================================================================= #
  def do_show_everything
    add(:everything) # Simply add everything.
  end; alias do_show_full_information do_show_everything # === do_show_full_information
       alias show_full_information    do_show_everything # === show_full_information

  # ========================================================================= #
  # === gui?
  # ========================================================================= #
  def gui?
    @runmode == :gui
  end

  # ========================================================================= #
  # === commandline?
  #
  # Whether we run in the "commandline mode" or whether we run in the
  # GUI or WWW/HTML mode.
  # ========================================================================= #
  def commandline?
    @runmode == :commandline
  end

  # ========================================================================= #
  # === be_silent
  # ========================================================================= #
  def be_silent
    @be_silent = true
  end

  # ========================================================================= #
  # === set_be_silent
  # ========================================================================= #
  def set_be_silent(i = true)
    @be_silent = i
  end

  # ========================================================================= #
  # === be_silent?
  # ========================================================================= #
  def be_silent?
    @be_silent
  end

  # ========================================================================= #
  # === start_the_sinatra_interface
  # ========================================================================= #
  def start_the_sinatra_interface
    require 'environment_information/www/sinatra_interface.rb'
    ::EnvironmentInformation.start_sinatra_interface
  end

  # ========================================================================= #
  # === xorg_components?
  # ========================================================================= #
  def xorg_components?
    ::EnvironmentInformation.xorg_components?
  end; alias return_all_xorg_components xorg_components? # === return_all_xorg_components

  # ========================================================================= #
  # === do_show_only_the_xorg_components
  #
  # This method will essentially clear the old dataset before adding
  # all xorg-components to the display-part of this class.
  #
  # The components that are appended here, are defined in the file
  # constants/array_tracked_components.rb - so if you wish to add
  # new xorg-related entries, you should modify the entries in that
  # .rb file.
  # ========================================================================= #
  def do_show_only_the_xorg_components
    clear_old_dataset
    do_not_show_the_ruby_components
    add(:all_xorg_components)
  end

  # ========================================================================= #
  # === do_not_display_the_result
  # ========================================================================= #
  def do_not_display_the_result
    @display_result = false
  end

  # ========================================================================= #
  # === bit_type?
  # ========================================================================= #
  def bit_type?
    ::EnvironmentInformation.send(__method__)
  end

  # ========================================================================= #
  # === cpu_model?
  # ========================================================================= #
  def cpu_model?
    ::EnvironmentInformation.send(__method__)
  end

  # ========================================================================= #
  # === cflags_in_use?
  # ========================================================================= #
  def cflags_in_use?
    ::EnvironmentInformation.send(__method__)
  end

  # ========================================================================= #
  # === assign_components_for_the_short_format
  #
  # We will only display 6 components when we use the short variant.
  # ========================================================================= #
  def assign_components_for_the_short_format
    clear_old_dataset
    dont_show_ruby_stuff
    add(%i(
      operating_system
      operating_system_bit_type
      cpuinfo
      cflags
      RAM
      screen_resolution
    ))
  end

  # ========================================================================= #
  # === do_compare_the_program_version
  #
  # This method can be used if the user wishes to also compare the
  # program version of the installed programs.
  # ========================================================================= #
  def do_compare_the_program_version(
      be_verbose = true
    )
    if be_verbose
      @array_show_this_to_the_user <<
        'The program versions will also be compared.'
    end
    @compare_program_versions = true
  end

  # ========================================================================= #
  # === may_we_try_to_use_rbt?
  # ========================================================================= #
  def may_we_try_to_use_rbt?
    @may_we_try_to_use_rbt
  end

  # ========================================================================= #
  # === compare_program_version?
  # ========================================================================= #
  def compare_program_version?
    @compare_program_version
  end

  # ========================================================================= #
  # === write_what_into
  # ========================================================================= #
  def write_what_into(what, into)
    ::EnvironmentInformation.write_what_into(what, into)
  end

  # ========================================================================= #
  # === do_show_only_the_operating_system
  # ========================================================================= #
  def do_show_only_the_operating_system
    clear_old_dataset
    add(:operating_system)
  end

  # ========================================================================= #
  # === do_sort_alphabetically                                     (sort tag)
  # ========================================================================= #
  def do_sort_alphabetically
    _ = display_which_components?.sort_by {|name, version| name }
    set_main_array(_)
  end

  # ========================================================================= #
  # === enable_sort_alphabetically
  # ========================================================================= #
  def enable_sort_alphabetically
    @sort_alphabetically = true
  end

  # ========================================================================= #
  # === do_display_in_a_short_format
  #
  # The short-format means that we will use only a compact set of
  # programs to display. For example, "make" and "bash" will not be
  # displayed, neither "binutils" or "ruby".
  # ========================================================================= #
  def do_display_in_a_short_format
    @display_everything_in_short_format = true
  end

  # ========================================================================= #
  # === show_the_registered_components
  # ========================================================================= #
  def show_the_registered_components(
      i = ARRAY_TRACKED_PROGRAMS
    )
    e
    e 'The following programs will be tracked:'
    e
    i.each_with_index {|this_program, index| index += 1
      padded_index = '  '+(index.to_s+') ').rjust(5)
      colourized_and_padded_index = seagreen(padded_index)
      e colourized_and_padded_index+
        steelblue(this_program)
    }
    e
  end

  # ========================================================================= #
  # === return_every_registered_component
  # ========================================================================= #
  def return_every_registered_component
    array = ::EnvironmentInformation.tracked_programs?+
            ::EnvironmentInformation.tracked_non_programs?+
            ::EnvironmentInformation.science_cluster?+
            ::EnvironmentInformation.xorg_components?
    array = array.flatten.uniq
    # ======================================================================= #
    # === The mate-desktop
    #
    # Since as of 31.03.2019 we will also try to show the mate-desktop
    # components if the RBT project is available/installed.
    # ======================================================================= #
    if is_rbt_available?
      if RBT.const_defined?(:ReportMateDesktopVersion)
        # =================================================================== #
        # Keep in mind that this Array is nested, so the name is
        # included as well as the version.
        # =================================================================== #
        _ = RBT.return_mate_desktop_version_array
        _.each {|a,b|
          array << [a, b]
        }
      end
    end
    return array # Return our findings.
  end

  # ========================================================================= #
  # === show_n_registered_entries
  #
  # Invoke this via:
  #
  #   envi --n_entries
  #
  # ========================================================================= #
  def show_n_registered_entries
    e "#{true_rev}The EnvironmentInformation project contains "\
      "#{sfancy(::EnvironmentInformation.tracked_programs?.size)} "\
      "#{true_rev}registered entries."
  end

  # ========================================================================= #
  # === set_runmode_gui
  # ========================================================================= #
  def set_runmode_gui
    @runmode = :gui
  end

  # ========================================================================= #
  # === set_runmode_html
  # ========================================================================= #
  def set_runmode_html
    @runmode = :html
  end

  # ========================================================================= #
  # === display_these_components?
  #
  # Note that @display_these_components is a Hash.
  # ========================================================================= #
  def display_these_components?
    @array_display_these_components
  end; alias components?                  display_these_components? # === components?
       alias main_dataset?                display_these_components? # === main_dataset?
       alias main_entry?                  display_these_components? # === main_entry?
       alias dataset?                     display_these_components? # === dataset?
       alias result                       display_these_components? # === result
       alias result?                      display_these_components? # === result?
       alias display_which_components?    display_these_components? # === display_which_components?
       alias array_obtain_these_programs? display_these_components? # === array_obtain_these_programs?
       alias array_with_entries?          display_these_components? # === array_with_entries?
       alias main_array?                  display_these_components? # === main_array?
       alias programs?                    display_these_components? # === programs?
       alias show_these_components?       display_these_components? # === show_these_components?

  # ========================================================================= #
  # === dataset_as_string
  #
  # This method must return the dataset in String format. That String
  # must already be properly "formatted".
  # ========================================================================= #
  def dataset_as_string
    _ = ''.dup # Put the information onto that String here.
    main_dataset?.each {|entry|
      if entry.is_a? Array
        entry = entry.first # The second entry is ignored in that event.
      end
      # ===================================================================== #
      # Before we can use .send() we have to check whether EnvironmentInformation
      # actually responds to that method. If not then we will simply skip
      # this snippet for now; but this may have to be changed at some
      # point in the future, to more elegantly handle failure. (Sep 2019).
      # ===================================================================== #
      use_this_method = "return_version_of_#{entry}".to_sym
      if ::EnvironmentInformation.respond_to? use_this_method
        program_version = ::EnvironmentInformation.send(
          use_this_method
        )
        entry = "  #{entry}:"
        _ << "#{entry} #{program_version}#{N}"
      end
    }
    return _ # And return the generated String.
  end; alias string?      dataset_as_string # === string?
       alias string       dataset_as_string # === string
       alias main_string? dataset_as_string # === main_string?
       alias stringified  dataset_as_string # === stringified

  # ========================================================================= #
  # === is_on_roebe?
  # ========================================================================= #
  def is_on_roebe?
    ::EnvironmentInformation.is_on_roebe?
  end

  # ========================================================================= #
  # === do_rename_kde_konsole
  #
  # This will attempt to rename the KDE Konsole tab, but only if we are
  # on roebe.
  # ========================================================================= #
  def do_rename_kde_konsole(
      use_this_title = 'Environment Information'
    )
    begin
      require 'roebe/requires/require_kde_konsole.rb'
    rescue LoadError; end  
    if @try_to_rename_the_kde_konsole_tab and
       is_on_roebe? and
       Object.const_defined?(:Roebe) and
       Roebe.const_defined?(:KdeKonsole)
      Roebe.rename_konsole(use_this_title)
    end
  end

  # ========================================================================= #
  # === is_a_registered_component?
  # ========================================================================= #
  def is_a_registered_component?(i)
    i = i.to_sym unless i.is_a? Symbol
    ::EnvironmentInformation.is_this_component_included?(i)
  end; alias is_an_allowed_entry? is_a_registered_component? # === is_an_allowed_entry?

  # ========================================================================= #
  # === do_report_the_remote_urls
  # ========================================================================= #
  def do_report_the_remote_urls
    @report_the_remote_urls = true
  end

  # ========================================================================= #
  # === do_not_run_already
  # ========================================================================= #
  def do_not_run_already
    @run_already = false
  end; alias do_not_run do_not_run_already # === do_not_run

  # ========================================================================= #
  # === start_gtk_component
  #
  # To invoke this method from the commandline, do:
  #
  #   envi --start-gtk
  #
  # ========================================================================= #
  def start_gtk_component
    require 'environment_information/gui/gtk/bindings.rb'
    ::EnvironmentInformation.run_gtk
    @do_exit_the_program = true
  end

  # ========================================================================= #
  # === do_exit_the_program
  # ========================================================================= #
  def do_exit_the_program
    @do_exit_the_program = true
  end

  # ========================================================================= #
  # === return_default_programs_on_linux
  #
  # This method should return the "main" programs on a linux computer,
  # the most important entries.
  # ========================================================================= #
  def return_default_programs_on_linux
    ARRAY_DEFAULT_PROGRAMS_ON_LINUX
  end; alias return_default_programs_on_a_linux_computer return_default_programs_on_linux # === return_default_programs_on_a_linux_computer

  # ========================================================================= #
  # === report_version
  # ========================================================================= #
  def report_version
    require 'environment_information/toplevel_methods/menu.rb'
    ::EnvironmentInformation.report_version
    @do_exit_the_program = true
  end

  # ========================================================================= #
  # === cd                                                           (cd tag)
  #
  # A shorter variant for the change-directory action.
  # ========================================================================= #
  def cd(i)
    ::EnvironmentInformation.cd(i)
  end

  # ========================================================================= #
  # === ram?
  # ========================================================================= #
  def ram?
    ::EnvironmentInformation.send(__method__)
  end

  # ========================================================================= #
  # === screen_resolution?
  # ========================================================================= #
  def screen_resolution?
    ::EnvironmentInformation.send(__method__)
  end

  # ========================================================================= #
  # === return_version_of_boost?
  # ========================================================================= #
  def return_version_of_boost?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_boost return_version_of_boost? # == return_version_of_boost

  # ========================================================================= #
  # === return_version_of_awk?
  # ========================================================================= #
  def return_version_of_awk?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_awk return_version_of_awk? # == return_version_of_awk

  # ========================================================================= #
  # === return_version_of_binutils?
  # ========================================================================= #
  def return_version_of_binutils?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_binutils return_version_of_binutils? # == return_version_of_binutils

  # ========================================================================= #
  # === return_version_of_coreutils?
  # ========================================================================= #
  def return_version_of_coreutils?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_coreutils return_version_of_coreutils? # == return_version_of_coreutils

  # ========================================================================= #
  # === return_version_of_diffutils?
  # ========================================================================= #
  def return_version_of_diffutils?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_diffutils return_version_of_diffutils? # == return_version_of_diffutils

  # ========================================================================= #
  # === return_version_of_gcc?
  # ========================================================================= #
  def return_version_of_gcc?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_gcc return_version_of_gcc? # == return_version_of_gcc

  # ========================================================================= #
  # === return_version_of_linux_kernel?
  # ========================================================================= #
  def return_version_of_linux_kernel?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_linux_kernel return_version_of_linux_kernel? # == return_version_of_linux_kernel

  # ========================================================================= #
  # === is_this_program_included?
  # ========================================================================= #
  def is_this_program_included?(i)
    ::EnvironmentInformation.is_this_program_included?(i)
  end

  # ========================================================================= #
  # === return_version_of_glibc?
  # ========================================================================= #
  def return_version_of_glibc?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_glibc return_version_of_glibc? # == return_version_of_glibc

  # ========================================================================= #
  # === return_version_of_intltool?
  # ========================================================================= #
  def return_version_of_intltool?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_intltool return_version_of_intltool? # == return_version_of_intltool

  # ========================================================================= #
  # === return_version_of_grep?
  # ========================================================================= #
  def return_version_of_grep?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_grep return_version_of_grep? # == return_version_of_grep

  # ========================================================================= #
  # === return_version_of_gnupg?
  # ========================================================================= #
  def return_version_of_gnupg?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_gnupg return_version_of_gnupg? # == return_version_of_gnupg

  # ========================================================================= #
  # === return_version_of_bison?
  # ========================================================================= #
  def return_version_of_bison?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_bison return_version_of_bison? # == return_version_of_bison

  # ========================================================================= #
  # === return_version_of_flex?
  # ========================================================================= #
  def return_version_of_flex?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_flex return_version_of_flex? # == return_version_of_flex

  # ========================================================================= #
  # === is_rbt_available?                                           (rbt tag)
  #
  # Query whether the user has the necessary code from the RBT project
  # installed.
  # ========================================================================= #
  def is_rbt_available?
    Object.const_defined? :RBT
  end

  # ========================================================================= #
  # === operating_system?
  #
  # To quickly test this method, try:
  #
  #   EnvironmentInformation::EnvironmentInformation.new { :do_not_run_yet }.operating_system?
  #
  # ========================================================================= #
  def operating_system?
    ::EnvironmentInformation.send(__method__)
  end; alias return_version_of_operating_system operating_system? # === return_version_of_operating_system

end; end