#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/report.rb'
# =========================================================================== #
require 'environment_information/toplevel_methods/is_on_roebe.rb'

module EnvironmentInformation

class EnvironmentInformation # === EnvironmentInformation::EnvironmentInformation

  begin
    require 'rbt/utility_scripts/report_mate_desktop_version.rb'
  rescue LoadError; end

  begin
    require 'rbt/toplevel_methods/swift_return_version_of_this_program.rb'
  rescue LoadError
    if ::EnvironmentInformation.is_on_roebe?
      e 'The file rbt/toplevel_methods/swift_return_version_of_this_program.rb is not available.'
    end
  end

  begin
    require 'rbt/toplevel_methods/find_url_for.rb'
  rescue LoadError
    if ::EnvironmentInformation.is_on_roebe?
      e 'The file rbt/toplevel_methods/find_url_for.rb is not available.'
    end
  end

  # ========================================================================= #
  # === clear_toplevel_hash
  # ========================================================================= #
  def clear_toplevel_hash
    ::EnvironmentInformation.clear_hash
  end

  # ========================================================================= #
  # === add_this_to_the_toplevel_hash
  # ========================================================================= #
  def add_this_to_the_toplevel_hash(a, b)
    ::EnvironmentInformation.hash?[a] = b
  end

  # ========================================================================= #
  # === report                                                   (report tag)
  # ========================================================================= #
  def report
    if @array_display_these_components.empty?
      opn; e 'There are no components that can be displayed.' # And this ends here.
    else
      e
      clear_toplevel_hash
      # ===================================================================== #
      # Iterate over the main components next.
      # ===================================================================== #
      @array_display_these_components.each {|this_program, version|
        if version.nil?
          version = ::EnvironmentInformation.version_for?(this_program)
        end
        if version.nil?
          register_unavailable_program(this_program)
        end
        add_this_to_the_toplevel_hash(this_program, version)
        # =================================================================== #
        # Next do some sanitizing.
        # =================================================================== #
        case this_program.to_s
        # ===================================================================== #
        # === operating_system
        # ===================================================================== #
        when /^operating(_|-)?system$/i,
             /^operating(_|-)?system(_|-)?in(_|-)?use$/i
          this_program = 'Operating system'
        # ===================================================================== #
        # === operating_system_bit_type
        # ===================================================================== #
        when /^operating(_|-)?system(_|-)?bit(_|-)?type$/i
          this_program = 'Operating system bit type'
        # ===================================================================== #
        # === ram
        # ===================================================================== #
        when /^ram$/i
          this_program = 'RAM'
        # ===================================================================== #
        # === cflags
        # ===================================================================== #
        when /^cflags$/i
          this_program = 'cflags in use'
        # ===================================================================== #
        # === screen_resolution
        # ===================================================================== #
        when /^screen( |_)?resolution$/i
          this_program = 'Screen Resolution'
        # =================================================================== #
        # === cpflags
        # =================================================================== #
        when /^cflags$/i,
             /^CFLAGS in use$/i
          this_program = 'CFLAGS in use'
        # =================================================================== #
        # === cpuinfo
        # =================================================================== #
        when 'cpuinfo'
          this_program = 'CPU Model'
        end
        left_side  = ::EnvironmentInformation.prepare_left_side(this_program.to_s.dup)
        right_side = ::EnvironmentInformation.prepare_right_side(version.to_s.ljust(6)).to_s
        if version.nil?
          right_side = dimgray('[Not found or installed.]')
        end
        this_program = this_program.dup if this_program.frozen?
        case this_program.to_s.strip
        # =================================================================== #
        # === Screen Resolution
        # =================================================================== #
        when /^RAM$/
          right_side = right_side.dup if right_side.frozen?
          right_side << orange('MB RAM')
        # =================================================================== #
        # === Screen Resolution
        # =================================================================== #
        when /^Screen Resolution$/
          if right_side.include?('x') and use_colours?
            splitted = right_side.split('x')
            right_side = colourize_this_in_the_right_side_colour(splitted.first)+
                         royalblue('x')+
                         colourize_this_in_the_right_side_colour(splitted.last)
          end
        end
        if right_side.include? 'steelblue'
          # ================================================================= #
          # An ad-hoc solution - we assume it was not found.
          # ================================================================= #
          right_side = '[Not found.]'
          if use_colours?
            right_side = orange(right_side)
          end
        end
        # =================================================================== #
        # === @report_the_remote_urls
        #
        # Next, honour @report_the_remote_urls if it is true.
        # =================================================================== #
        if @report_the_remote_urls
          if Object.const_defined?(:RBT) and
             RBT.respond_to?(:remote_url_for?)
            version = ''.dup
            _ = this_program.to_s.delete('-').downcase.strip
            if RBT.does_include?(_)
              remote_url = RBT.remote_url_for?(_, :return_as_string)
              if is_a_registered_component? _
                if version.frozen?
                  version = version.dup
                end
                version << lightgreen(remote_url)
                right_side = right_side.dup if right_side.frozen?
                right_side << " #{olive('→')} #{version}"
              end
            end
          end
        end
        e left_side+right_side
        _ = this_program.to_s.delete(':')
        # =================================================================== #
        # Next compare the program versions.
        # =================================================================== #
        if @compare_program_versions and
           may_we_try_to_use_rbt? and
           is_rbt_available?      and
           RBT.does_include?(_)
          registered_local_version = RBT.swift_return_version_of_this_program(_.to_sym).to_s
          # ================================================================= #
          # We ignore gtk2 when it comes to the version-check.
          # ================================================================= #
          next if this_program == :gtk2
          # ================================================================= #
          # Next use Gem::Version to compare them, but not if the version
          # is nil.
          # ================================================================= #
          unless version.nil?
            # =============================================================== #
            # The .delete('v') part is specifically to workaround node.
            # =============================================================== #
            if Gem::Version.new(registered_local_version.delete('v').tr('_','.')) > Gem::Version.new(version)
              e slateblue('     ^^^^^ This program could be upgraded, '\
                'to the version ')+
                steelblue(registered_local_version)
            end
          end
        end
      }
      e
    end
  end; alias display                     report # === display
       alias report_result               report # === report_result
       alias display_the_components      report # === display_the_components
       alias display_the_dataset         report # === display_the_dataset
       alias work_through_the_programs   report # === work_through_the_programs
       alias build_up_the_main_string    report # === build_up_the_main_string
       alias display_the_main_components report # === build_up_the_main_string

  # ========================================================================= #
  # === e                                                             (e tag)
  #
  # The e() method is the general output-method for this class.  
  # ========================================================================= #
  def e(i = '')
    if @display_everything_in_short_format
      # ===================================================================== #
      # Display the results in a single line.
      # ===================================================================== #
      print "#{i}, "
    else
      puts i
    end unless @be_silent
  end

end; end