#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/class/reset.rb'
# =========================================================================== #
module EnvironmentInformation

class EnvironmentInformation # === EnvironmentInformation::EnvironmentInformation

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    # ======================================================================= #
    # === @store_the_results_into_local_files
    #
    # If the following variable is set to true then the project will
    # generate local files too, e. g. yaml files and what not.
    # ======================================================================= #
    @store_the_results_into_local_files = true
    # ======================================================================= #
    # === @use_ascii_table
    #
    # Whether to use an ASCII table or whether we will not:
    # ======================================================================= #
    @use_ascii_table = false
    # ======================================================================= #
    # === @table
    # ======================================================================= #
    @table = nil # The ascii table.
    # ======================================================================= #
    # === @array_unavailable_programs
    #
    # Programs which could not be found can be registered into the following
    # Array.
    # ======================================================================= #
    @array_unavailable_programs = []
    # ======================================================================= #
    # === @runmode
    #
    # The @runmode variable can be :commandline or :gui or :www.
    # ======================================================================= #
    @runmode = :commandline
    # ======================================================================= #
    # === @show_everything
    #
    # If the following instance variable is set to true then this class
    # will try to show every registered (and thus, available) component.
    #
    # By default this is not wanted, so it is disabled. The user has to
    # specifically enable this option via the commandline, if so
    # desired, and thus overrule this default value.
    # ======================================================================= #
    @show_everything = false
    # ======================================================================= #
    # === @display_result
    #
    # If the following instance variable is true, which is the case by
    # default, then this class will report to the user on the commandline.
    #
    # If it is set to false then nothing will be displayed; this is
    # useful when you only want to obtain the dataset, without
    # showing anything to the user.
    # ======================================================================= #
    @display_result = true
    # ======================================================================= #
    # === @be_silent
    #
    # By default, this class is not silent, meaning that it will display
    # information to the user.
    # ======================================================================= #
    @be_silent = false
    # ======================================================================= #
    # === @array_these_programs_not_up_to_date
    #
    # This Array can be used to save into a local file which programs
    # are not up to date.
    # ======================================================================= #
    if is_on_roebe?
      @array_these_programs_not_up_to_date = []
    end
    # ======================================================================= #
    # === @compare_program_versions
    #
    # This instance variable can be used to also compare the program
    # versions, if the RBT project is available.
    #
    # By default this will not be done, though.
    # ======================================================================= #
    @compare_program_versions = false
    # ======================================================================= #
    # === @may_we_try_to_use_rbt
    #
    # Whether we may query RBT for additional help or not. By default
    # we will try to make use of RBT.
    # ======================================================================= #
    @may_we_try_to_use_rbt = true
    # ======================================================================= #
    # === @array_show_this_to_the_user
    #
    # The following Array can be used to show messages to the user.
    # ======================================================================= #
    @array_show_this_to_the_user = []
    # ======================================================================= #
    # === @array_display_these_components
    #
    # This Array will display the components on the commandline.
    #
    # Take note that the order is important: the entries that appear first
    # will be displayed earlier. In other words: first entries will be
    # shown first as well.
    # ======================================================================= #
    @array_display_these_components = []
    # ======================================================================= #
    # === @sort_alphabetically
    #
    # Whether to sort the main Hash alphabetically or not.
    # ======================================================================= #
    @sort_alphabetically = false
    # ======================================================================= #
    # === @show_ruby_version_and_gem_version
    #
    # This instance variable determines whether class EnvironmentInformation
    # will display the ruby version and the gem version.
    #
    # By default we will do so, but there may be situations where this
    # may be unwanted, or not a good idea, such as in a minimal system
    # where no ruby is running, or if the user only wants to display
    # very little information.
    # ======================================================================= #
    @show_ruby_version_and_gem_version = true
    # ======================================================================= #
    # === @display_everything_in_short_format
    #
    # This variable determines whether we will use a compact-display or
    # whether there will be one-entry-per-program instead.
    # ======================================================================= #
    @display_everything_in_short_format = false
    # ======================================================================= #
    # === @generate_a_html_file
    #
    # If the next instance variable is set to true then a html file will
    # be generated. By default this will not happen, though.
    # ======================================================================= #
    @generate_a_html_file = false
    # ======================================================================= #
    # === @try_to_rename_the_kde_konsole_tab
    #
    # The following instance variable will determine as to whether we will
    # try to make use of the KDE Konsole and rename the tab of the konsole
    # there.
    #
    # Since as of October 2018 we will not use the KDE konsole by default
    # anymore. This may change at a later moment in time, though.
    # ======================================================================= #
    @try_to_rename_the_kde_konsole_tab = false
    # ======================================================================= #
    # === @run_already
    #
    # This variable will be true by default.
    # ======================================================================= #
    @run_already = true
    # ======================================================================= #
    # === @show_help
    #
    # If this variable is set to true, then we will only show help, then
    # exit the program.
    # ======================================================================= #
    @show_help = false
    # ======================================================================= #
    # === @report_the_remote_urls
    #
    # If this instance variable is set to true then the remote URLs will
    # be shown as well, on the commandline.
    # ======================================================================= #
    @report_the_remote_urls = false
    # ======================================================================= #
    # === @do_exit_the_program
    #
    # The following variable can determine when we exit from this class.
    # By default we will not exist early.
    # ======================================================================= #
    @do_exit_the_program = false
    # ======================================================================= #
    # Add the default programs on a linux computer.
    # ======================================================================= #
    add(
      return_default_programs_on_a_linux_computer
    )
  end

end; end