#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/colours/colours.rb'
# EnvironmentInformation.colour_for_the_left_side
# =========================================================================== #
module EnvironmentInformation

  begin
    require 'colours'
    # ======================================================================= #
    # === @use_colours
    # ======================================================================= #
    @use_colours = true
  rescue LoadError
    # ======================================================================= #
    # If the colours-gem is not available then we will not use colours.
    # The user could still overrule this at a later time, though.
    # ======================================================================= #
    @use_colours = false
  end

  # ========================================================================= #
  # === EnvironmentInformation.use_colours?
  # ========================================================================= #
  def self.use_colours?
    @use_colours
  end

  # ========================================================================= #
  # === EnvironmentInformation.set_use_colours
  #
  # Determine whether the EnvironmentInformation project will use
  # colours or whether it will not.
  # ========================================================================= #
  def self.set_use_colours(i = true)
    @use_colours = i
  end; self.instance_eval { alias use_colours= set_use_colours } # === EnvironmentInformation.use_colours=
  
  # ========================================================================= #
  # === EnvironmentInformation.do_use_colours
  # ========================================================================= #
  def self.do_use_colours
    @use_colours = i
  end

  # ========================================================================= #
  # === EnvironmentInformation.mediumpurple
  # ========================================================================= #
  def self.mediumpurple(i = '')
    if @use_colours
      i = ::Colours.mediumpurple(i)
    end
    i
  end

  # ========================================================================= #
  # === EnvironmentInformation.lightseagreen
  # ========================================================================= #
  def self.lightseagreen(i = '')
    if @use_colours
      i = ::Colours.lightseagreen(i)
    end
    i
  end

  # ========================================================================= #
  # === EnvironmentInformation.steelblue
  # ========================================================================= #
  def self.steelblue(i = '')
    if @use_colours
      i = ::Colours.steelblue(i)
    end
    i
  end

  # ========================================================================= #
  # === EnvironmentInformation.royalblue
  # ========================================================================= #
  def self.royalblue(i = '')
    if @use_colours
      i = ::Colours.royalblue(i)
    end
    i
  end

  # ========================================================================= #
  # === EnvironmentInformation.sfile
  # ========================================================================= #
  def self.sfile(i = '')
    if @use_colours
      ::Colours.sfile(i)
    else
      i
    end
  end

  # ========================================================================= #
  # === EnvironmentInformation.rev
  # ========================================================================= #
  def self.rev
    ::Colours.rev
  end

  # ========================================================================= #
  # === EnvironmentInformation.colour_for_the_left_side
  #
  # This could also be :limegreen rather than :forestgreen or vice versa.
  # ========================================================================= #
  def self.colour_for_the_left_side
    :forestgreen
  end

  # ========================================================================= #
  # === EnvironmentInformation.colour_for_the_right_side
  # ========================================================================= #
  def self.colour_for_the_right_side
    :steelblue
  end

end