#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/constants/temp_directory.rb'
# =========================================================================== #
module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation::TEMP_DIRECTORY
  #
  # Where to store files. Must have a trailing '/' character.
  #
  # The environment variable called MY_TEMP_DIR can overrule all
  # other guesses here, but it is not always available, e. g.
  # in a .cgi file it may not be available.
  # ========================================================================= #
  if ENV.has_key? 'MY_TEMP_DIR'
    TEMP_DIRECTORY = ENV['MY_TEMP_DIR'].to_s
  elsif File.directory? '/home/Temp/' # ← My main directory.
    TEMP_DIRECTORY = '/home/Temp/'
  elsif File.directory? '/Depot/Temp/'
    TEMP_DIRECTORY = '/Depot/Temp/'
  else
    TEMP_DIRECTORY = '/tmp/' # ← "Rescue" for most other users on linux.
  end

  # ========================================================================= #
  # === @temp_directory
  # ========================================================================= #
  @temp_directory = TEMP_DIRECTORY

  # ========================================================================= #
  # === EnvironmentInformation.temp_directory?
  #
  # This method will refer to the temp-directory in use for this project.
  # ========================================================================= #
  def self.temp_directory?(
      i = @temp_directory
    )
    i.squeeze('/').dup
  end; self.instance_eval { alias temp_dir? temp_directory? } # === EnvironmentInformation.temp_dir?

end