#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/misc_components/cpuinfo.rb'
# =========================================================================== #
require 'environment_information/constants/newline.rb'
require 'environment_information/constants/error_line.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.cpuinfo?                         (cpuinfo tag)
  #
  # This method currently relies on /proc/cpuinfo, thus favouring
  # Linux-based systems.
  #
  # It will become the display part of "CPU Model".
  # ========================================================================= #
  def self.cpuinfo?
    result = '/proc/cpuinfo'
    if File.exist? result
      readlines = File.readlines(result)
      _ = readlines.grep(/model name/).first.chomp
      splitted = _.split(':')
      _ = splitted[1].strip # => "model name\t: AMD Sempron(tm) 145 Processor"
      # ===================================================================== #
      # Next, add the amount of cores. We use "nproc" for this but "lscpu"
      # also contains this information. The following regex could be used
      # for lscpu:
      #
      #   Core\(s\) per socket:    (.+)
      #
      # Ok, we use lscpu for now.
      # ===================================================================== #
      n_cores = `nproc --all`.strip
      case n_cores.to_s
      when '1'
        n_cores << ' core'  # Singular.
      else
        n_cores << ' cores' # Plural then.
      end
      _ << ", #{n_cores}" unless n_cores.empty?
      result = _
    else
      result = nil
    end
    result
  end; self.instance_eval { alias append_operating_system_cpuinfo cpuinfo? } # === EnvironmentInformation.append_operating_system_cpuinfo
       self.instance_eval { alias cpu_model                       cpuinfo? } # === EnvironmentInformation.cpu_model
       self.instance_eval { alias cpu_model?                      cpuinfo? } # === EnvironmentInformation.cpu_model?
       self.instance_eval { alias return_version_of_cpuinfo       cpuinfo? } # === EnvironmentInformation.return_version_of_cpuinfo
       self.instance_eval { alias cpuinfo                         cpuinfo? } # === EnvironmentInformation.cpuinfo

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.cpuinfo?
end