#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/misc_components/operating_system.rb'
# =========================================================================== #
require 'environment_information/constants/newline.rb'
require 'environment_information/constants/error_line.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.operating_system                      (os tag)
  #
  # This method is also known as "bit type".
  #
  # The return value of this method may be a String such as this one here:
  #
  #   "GNU/Linux"
  #
  # ========================================================================= #
  def self.operating_system
    cmd = 'uname -mo'
    result = `#{cmd}`.chomp
    if result.start_with? 'GNU'
      # ===================================================================== #
      # The next part removes 'GNU/' specifically. It is a simpler name
      # than "GNU Linux" and variants, just as "BSD" is simpler than
      # e. g. "FreeBSD". You can reason that it may be "less accurate",
      # but it makes the notification-part of this gem simpler.
      # ===================================================================== #
      result.sub!(/^GNU\//,'') # Experimental as of Sep 2019.
    end
    if result.include? ' '
      result = result.split(' ').last
    end
    result
  end; self.instance_eval { alias operating_system?                         operating_system } # === EnvironmentInformation.operating_system?
       self.instance_eval { alias operating_system_information?             operating_system } # === EnvironmentInformation.operating_system_information
       self.instance_eval { alias add_operating_system_information          operating_system } # === EnvironmentInformation.add_operating_system_information
       self.instance_eval { alias append_operating_system_in_use            operating_system } # === EnvironmentInformation.append_operating_system_in_use
       self.instance_eval { alias add_os_information                        operating_system } # === EnvironmentInformation.add_os_information
       self.instance_eval { alias operating_system_in_use?                  operating_system } # === EnvironmentInformation.operating_system_in_use?
       self.instance_eval { alias return_version_of_operating_system        operating_system } # === EnvironmentInformation.return_version_of_operating_system
       self.instance_eval { alias return_version_of_operating_system_in_use operating_system } # === EnvironmentInformation.return_version_of_operating_system_in_use

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.operating_system
end