#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/misc_components/rubygems_installation_directory.rb'
# =========================================================================== #
require 'environment_information/constants/newline.rb'
require 'environment_information/constants/error_line.rb'
require 'environment_information/toplevel_methods/prefix_to_use.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.rubygems_installation_directory
  #
  # This method will return the path to the rubygems installation
  # directory, if possible (if it exists).
  #
  # As this ought to be a directory, we will ensure that a trailing '/'
  # character exists.
  # ========================================================================= #
  def self.rubygems_installation_directory(
      prefix_to_use = @prefix_to_use
    )
    result = `#{prefix_to_use}gem env #{ERROR_LINE}` # We need the 2>&1 to check if gem is available.
    unless result.include? 'not found'
      # ===================================================================== #
      # Apply a regex next.
      # ===================================================================== #
      path_to_the_rubygem_directory = result.to_s.scan(
        /INSTALLATION DIRECTORY: (.+)/ # Obtain the proper match here.
      ).flatten.first.to_s
      if File.directory? path_to_the_rubygem_directory
        path_to_the_rubygem_directory << '/'
      end unless path_to_the_rubygem_directory.end_with? '/'
      result = path_to_the_rubygem_directory
    else
      result = nil
    end
    result
  end; self.instance_eval { alias add_rubygems_installation_directory_information   rubygems_installation_directory } # === EnvironmentInformation.add_rubygems_installation_directory_information
       self.instance_eval { alias add_rubygem_directory_information                 rubygems_installation_directory } # === EnvironmentInformation.add_rubygem_directory_information
       self.instance_eval { alias append_rubygem_installation_directory             rubygems_installation_directory } # === EnvironmentInformation.append_rubygem_installation_directory
       self.instance_eval { alias return_version_of_rubygems_installation_directory rubygems_installation_directory } # === EnvironmentInformation.return_version_of_rubygems_installation_directory

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.rubygems_installation_directory
end