#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/misc_components/screen_resolution.rb'
# =========================================================================== #
require 'environment_information/constants/newline.rb'
require 'environment_information/constants/error_line.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.screen_resolution                 (screen tag)
  #
  # This method will typically make use of xdpyinfo first, as it also
  # works in a .cgi environment. For non-cgi environments, on linux,
  # we could use xrandr, which is a bit more elegant.
  #
  # On success, the method here will return a String such as "1920x1080".
  # ========================================================================= #
  def self.screen_resolution
    result = '(unknown)' # This is the default return-value, then.
    # ======================================================================= #
    # === Check for linux as host OS first
    # ======================================================================= #
    if RUBY_PLATFORM.downcase.include? 'linux'
      # ================================================================= #
      # We have to be careful as xdpyinfo may not be installed on the
      # given computer system.
      # ================================================================= #
      resolution = `xdpyinfo #{ERROR_LINE}`.scan(/dimensions:.+$/).first # The error here may be: "xdpyinfo:  unable to open display"
      if resolution and resolution.include? ':'
        result = resolution.split(':').last.strip.split('pixels').first.strip
      end
    # ======================================================================= #
    # === Else simply assume to be on windows here
    # ======================================================================= #
    else
      result = `wmic desktopmonitor get screenheight,screenwidth`
    end
    result
  end; self.instance_eval { alias screen_resolution?                  screen_resolution } # === EnvironmentInformation.screen_resolution?
       self.instance_eval { alias return_version_of_screen_resolution screen_resolution } # === EnvironmentInformation.return_version_of_screen_resolution

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.screen_resolution
end