#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# Code in this file is tasked to autogenerate the relevant methods for use
# in the toplevel EnvironmentInformation namespace.
# =========================================================================== #
# require 'environment_information/toplevel_methods/autogenerate_all_relevant_methods.rb'
# =========================================================================== #
require 'environment_information/constants/array_tracked_components.rb'
require 'environment_information/toplevel_methods/return_version_of_this_program.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.autogenerate_all_relevant_methods
  # ========================================================================= #
  def self.autogenerate_all_relevant_methods(
      i = ARRAY_TRACKED_PROGRAMS+
          ARRAY_XORG_COMPONENTS
    )
    i.flatten!
    # ======================================================================= #
    # Add some aliases which will be defined next.
    # ======================================================================= #
    hash_aliases = {
      atk:   :libatk,   # === EnvironmentInformation.return_version_of_libatk
      pcre2: :libpcre2, # === EnvironmentInformation.return_version_of_libpcre2
      pcre:  :libpcre,  # === EnvironmentInformation.return_version_of_libpcre
      zlib:  :libzlib   # === EnvironmentInformation.return_version_of_libzlib
    }
    # ======================================================================= #
    # Iterate over our Array next:
    # ======================================================================= #
    i.each {|this_program|
      case this_program
      when :kde
        this_program = :kde_frameworks
      end
      self.class.instance_eval {
        # =================================================================== #
        # To test this method, try to do:
        #
        #   EnvironmentInformation.return_version_of_valgrind # => "3.15.0"
        #
        # =================================================================== #
        this_method = "return_version_of_#{this_program}"
        define_method(this_method) {
          return_version_of_this_program(this_program)
        }
        if hash_aliases.has_key?(this_program)
          # ================================================================= #
          # To test this, try:
          #
          #   EnvironmentInformation.return_version_of_libzlib
          #
          # ================================================================= #
          use_this_as_the_alias_target = "return_version_of_#{hash_aliases[this_program]}"
          define_method(use_this_as_the_alias_target) {
            return_version_of_this_program(this_program)
          }
        end
      }
    }
  end

  autogenerate_all_relevant_methods # ← And call it at once.

  # ========================================================================= #
  # And add a few more aliases.
  # ========================================================================= #
  self.instance_eval { alias add_xz_information                                         return_version_of_xz }             # === EnvironmentInformation.add_xz_information
  self.instance_eval { alias append_xz_version                                          return_version_of_xz }             # === EnvironmentInformation.append_xz_version
  self.instance_eval { alias return_sed_version                                         return_version_of_sed }            # === EnvironmentInformation.return_sed_version
  self.instance_eval { alias return_version_of_makeinfo                                 return_version_of_texinfo }        # === EnvironmentInformation.return_version_of_makeinfo
  self.instance_eval { alias xfsprogs?                                                  return_version_of_xfsprogs }       # === EnvironmentInformation.xfsprogs?
  self.instance_eval { alias add_xfsprogs_information                                   return_version_of_xfsprogs }       # === EnvironmentInformation.add_xfsprogs_information
  self.instance_eval { alias return_version_of_lftp?                                    return_version_of_lftp }           # === EnvironmentInformation.return_version_of_lftp?
  self.instance_eval { alias add_llvm_information                                       return_version_of_llvm }           # === EnvironmentInformation.add_llvm_information
  self.instance_eval { alias append_llvm_version                                        return_version_of_llvm }           # === EnvironmentInformation.append_llvm_version
  self.instance_eval { alias add_mpc_information                                        return_version_of_mpc }            # === EnvironmentInformation.add_mpc_information
  self.instance_eval { alias append_mpc_version                                         return_version_of_mpc }            # === EnvironmentInformation.append_mpc_version
  self.instance_eval { alias try_to_show_re2c_version                                   return_version_of_re2c }           # === EnvironmentInformation.try_to_show_re2c_version
  self.instance_eval { alias append_re2c_version                                        return_version_of_re2c }           # === EnvironmentInformation.append_nasm_version
  self.instance_eval { alias return_version_of_pkg_config                               return_version_of_pkgconfig }      # === EnvironmentInformation.return_version_of_pkg_config
  self.instance_eval { alias add_readline_information                                   return_version_of_readline }       # === EnvironmentInformation.add_readline_information
  self.instance_eval { alias append_readline_version                                    return_version_of_readline }       # === EnvironmentInformation.add_readline_information
  self.instance_eval { alias return_version_of_liblibtasn1                              return_version_of_libtasn1 }       # === EnvironmentInformation.return_version_of_liblibtasn1
  self.instance_eval { alias return_version_of_liblibtasn                               return_version_of_libtasn1 }       # === EnvironmentInformation.return_version_of_liblibtasn
  self.instance_eval { alias return_version_of_grep?                                    return_version_of_grep }           # === EnvironmentInformation.return_version_of_grep?
  self.instance_eval { alias return_version_of_intltool?                                return_version_of_intltool }       # === EnvironmentInformation.return_version_of_intltool?
  self.instance_eval { alias glib?                                                      return_version_of_glib }           # === EnvironmentInformation.glib?
  self.instance_eval { alias add_gmp_information                                        return_version_of_gmp }            # === EnvironmentInformation.add_gmp_information
  self.instance_eval { alias append_gmp_version                                         return_version_of_gmp }            # === EnvironmentInformation.append_gmp_version
  self.instance_eval { alias kernel                                                     return_version_of_linux }          # === EnvironmentInformation.kernel
  self.instance_eval { alias linux_kernel                                               return_version_of_linux }          # === EnvironmentInformation.linux_kernel
  self.instance_eval { alias linux_kernel?                                              return_version_of_linux }          # === EnvironmentInformation.linux_kernel?
  self.instance_eval { alias return_information_about_the_component_called_linux_kernel return_version_of_linux }          # === EnvironmentInformation.return_information_about_the_component_called_linux_kernel
  self.instance_eval { alias append_kernel_version                                      return_version_of_linux }          # === EnvironmentInformation.append_kernel_version
  self.instance_eval { alias add_kernel_information                                     return_version_of_linux }          # === EnvironmentInformation.add_kernel_information
  self.instance_eval { alias return_version_of_linux_kernel                             return_version_of_linux }          # === EnvironmentInformation.return_version_of_linux_kernel
  self.instance_eval { alias return_version_of_linux_kernel?                            return_version_of_linux }          # === EnvironmentInformation.return_version_of_linux_kernel?
  self.instance_eval { alias boost?                                                     return_version_of_boost }          # === EnvironmentInformation.boost?
  self.instance_eval { alias return_version_of_boost?                                   return_version_of_boost }          # === EnvironmentInformation.return_version_of_boost?
  self.instance_eval { alias return_version_of_find                                     return_version_of_findutils }      # === EnvironmentInformation.return_version_of_find
  self.instance_eval { alias findutils?                                                 return_version_of_findutils }      # === EnvironmentInformation.findutils?
  self.instance_eval { alias awk?                                                       return_version_of_gawk }           # === EnvironmentInformation.awk?
  self.instance_eval { alias gawk?                                                      return_version_of_gawk }           # === EnvironmentInformation.gawk?
  self.instance_eval { alias return_information_about_the_component_called_awk          return_version_of_gawk }           # === EnvironmentInformation.return_information_about_the_component_called_awk
  self.instance_eval { alias return_version_of_awk?                                     return_version_of_gawk }           # === EnvironmentInformation.return_version_of_awk?
  self.instance_eval { alias return_version_of_awk                                      return_version_of_gawk }           # === EnvironmentInformation.return_version_of_awk
  self.instance_eval { alias append_gdkpixbuf_version                                   return_version_of_gdkpixbuf }      # === EnvironmentInformation.append_gdkpixbuf_version
  self.instance_eval { alias glibc?                                                     return_version_of_glibc }          # === EnvironmentInformation.glibc?
  self.instance_eval { alias glibc_version?                                             return_version_of_glibc }          # === EnvironmentInformation.glibc_version?
  self.instance_eval { alias return_version_of_glibc?                                   return_version_of_glibc }          # === EnvironmentInformation.return_version_of_glibc?
  self.instance_eval { alias ruby?                                                      return_version_of_ruby }           # === EnvironmentInformation.ruby?
  self.instance_eval { alias return_information_about_the_component_called_ruby         return_version_of_ruby }           # === EnvironmentInformation.return_information_about_the_component_called_ruby
  self.instance_eval { alias return_version_of_ruby?                                    return_version_of_ruby }           # === EnvironmentInformation.return_version_of_ruby?
  self.instance_eval { alias binutils?                                                  return_version_of_binutils }       # === EnvironmentInformation.binutils?
  self.instance_eval { alias return_version_of_binutils?                                return_version_of_binutils }       # === EnvironmentInformation.return_version_of_binutils?
  self.instance_eval { alias bison?                                                     return_version_of_bison }          # === EnvironmentInformation.bison?
  self.instance_eval { alias return_version_of_bison?                                   return_version_of_bison }          # === EnvironmentInformation.return_version_of_bison?
  self.instance_eval { alias return_version_of_flex?                                    return_version_of_flex }           # === EnvironmentInformation.return_version_of_flex?
  self.instance_eval { alias return_version_of_freetype2                                return_version_of_freetype }       # === EnvironmentInformation.return_version_of_freetype2
  self.instance_eval { alias evince?                                                    return_version_of_evince }         # === EnvironmentInformation.evince?
  self.instance_eval { alias return_version_of_evince?                                  return_version_of_evince }         # === EnvironmentInformation.return_version_of_evince?
  self.instance_eval { alias gcc?                                                       return_version_of_gcc }            # === EnvironmentInformation.gcc?
  self.instance_eval { alias return_information_about_the_component_called_gcc          return_version_of_gcc }            # === EnvironmentInformation.return_information_about_the_component_called_gcc
  self.instance_eval { alias return_version_of_gcc?                                     return_version_of_gcc }            # === EnvironmentInformation.return_version_of_gcc?
  self.instance_eval { alias return_version_of_R                                        return_version_of_r }              # === EnvironmentInformation.return_version_of_R
  self.instance_eval { alias coreutils?                                                 return_version_of_coreutils }      # === EnvironmentInformation.coreutils?
  self.instance_eval { alias return_version_of_coreutils?                               return_version_of_coreutils }      # === EnvironmentInformation.return_version_of_coreutils
  self.instance_eval { alias return_version_of_diffutils?                               return_version_of_diffutils }      # === EnvironmentInformation.return_version_of_diffutils?
  self.instance_eval { alias return_version_of_libexempi                                return_version_of_exempi }         # === EnvironmentInformation.return_version_of_libexempi
  self.instance_eval { alias return_version_of_gpg                                      return_version_of_gnupg }          # === EnvironmentInformation.return_version_of_gpg
  self.instance_eval { alias return_version_of_gnupg?                                   return_version_of_gnupg }          # === EnvironmentInformation.return_version_of_gnupg?
  self.instance_eval { alias return_version_of_bzip                                     return_version_of_bzip2 }          # === EnvironmentInformation.return_version_of_bzip
  self.instance_eval { alias return_version_of_llvm                                     return_version_of_clang }          # === EnvironmentInformation.return_version_of_llvm
  self.instance_eval { alias return_clang_version                                       return_version_of_clang }          # === EnvironmentInformation.return_clang_version
  self.instance_eval { alias kde5?                                                      return_version_of_kde_frameworks } # === EnvironmentInformation.kde5?
  self.instance_eval { alias add_kde5_information                                       return_version_of_kde_frameworks } # === EnvironmentInformation.add_kde5_information
  self.instance_eval { alias add_kde_information                                        return_version_of_kde_frameworks } # === EnvironmentInformation.add_kde_information
  self.instance_eval { alias return_information_about_the_component_called_rubygems     return_version_of_rubygems }       # === EnvironmentInformation.return_information_about_the_component_called_rubygems
  self.instance_eval { alias rubygems?                                                  return_version_of_rubygems }       # === EnvironmentInformation.rubygems?
  self.instance_eval { alias return_version_of_rubygems?                                return_version_of_rubygems }       # === EnvironmentInformation.return_version_of_rubygems?
  self.instance_eval { alias gtk3?                                                      return_version_of_gtk3 }           # === EnvironmentInformation.gtk3?
  self.instance_eval { alias gtk2?                                                      return_version_of_gtk2 }           # === EnvironmentInformation.gtk2?

end