#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/menu.rb'
# =========================================================================== #
require 'environment_information/toplevel_methods/e.rb'
require 'environment_information/version/version.rb'
require 'environment_information/colours/colours.rb'
require 'environment_information/colours/simp.rb'
require 'environment_information/constants/regex.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.menu
  # ========================================================================= #
  def self.menu(
      i = ARGV
    )
    if i.is_a? Array
      i.each {|entry| menu(entry) }
    else
      case i # case tag
      # ===================================================================== #
      # === envi --version?
      # ===================================================================== #
      when *ARRAY_VERSION
        report_version
      # ===================================================================== #
      # === envi --replay
      #
      # The replay-functionality can be used to display information
      # quickly, by reading the dataset from a local file.
      # ===================================================================== #
      when /^-?-?replay$/i
        require 'environment_information/toplevel_methods/replay_from_the_stored_file.rb'
        replay_from_the_stored_file
      # ===================================================================== #
      # === envi --compare
      # ===================================================================== #
      when *ARRAY_COMPARE_PROGRAM_VERSIONS
        require 'environment_information/toplevel_methods/show_all_available_components.rb'
        ::EnvironmentInformation.show_all_available_components(
          ['--compare-program-versions']
        )
      # ===================================================================== #
      # === envi --gui
      # ===================================================================== #
      when /^-?-?gui$/i,
           /^-?-?gtk$/i,
           /^-?-?start(-|_)?gtk$/i
        start_gtk_component
      else
        require 'environment_information/toplevel_methods/misc.rb'
        EnvironmentInformation.test
      end
    end
  end

  # ========================================================================= #
  # === EnvironmentInformation.report_version
  # ========================================================================= #
  def self.report_version
    e "#{rev}The version of the EnvironmentInformation project in "\
      "use is: #{simp(VERSION)}#{rev}"
  end

  # ========================================================================= #
  # === EnvironmentInformation.start_gtk_component
  #
  # We have to rescue the code so to notify the user what exactly
  # may have failed.
  # ========================================================================= #
  def self.start_gtk_component
    begin
      this_file = 'environment_information/gui/gtk/bindings.rb'
      require this_file
    rescue LoadError => error
      e "An error happened - file #{sfile(this_file)} could not be found."
      pp error
    end
    ::EnvironmentInformation.run_gtk
  end

end

if __FILE__ == $PROGRAM_NAME
  EnvironmentInformation.menu(ARGV)
end