#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/misc.rb'
# =========================================================================== #
require 'yaml'
require 'environment_information/constants/file_constants.rb'
require 'environment_information/project/project.rb'
require 'environment_information/toplevel_methods/e.rb'
require 'environment_information/toplevel_methods/is_on_roebe.rb'
require 'environment_information/toplevel_methods/register_this_component_is_missing.rb'
require 'environment_information/toplevel_methods/return_version_of_this_program.rb'
require 'environment_information/toplevel_methods/prefix_to_use.rb'
require 'environment_information/toplevel_methods/write_what_into.rb'
require 'environment_information/toplevel_methods/replay_from_the_stored_file.rb'
require 'environment_information/colours/colours.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === @hash_available_programs
  # ========================================================================= #
  @hash_available_programs = {}

  # ========================================================================= #
  # === EnvironmentInformation.hash?
  #
  # This method will return the Hash of all available programs.
  # ========================================================================= #
  def self.hash?
    @hash_available_programs
  end; self.instance_eval { alias hash hash? } # === EnvironmentInformation.hash

  # ========================================================================= #
  # === EnvironmentInformation.clear_hash
  # ========================================================================= #
  def self.clear_hash
    @hash_available_programs = {}
  end; self.instance_eval { alias clear_main_hash clear_hash } # === EnvironmentInformation.clear_main_hash

  # ========================================================================= #
  # === EnvironmentInformation.consider_storing_these_results_into_a_local_file
  #
  # This method will always take the dataset stored in the main
  # hash when saving the yaml file at hand.
  # ========================================================================= #
  def self.consider_storing_these_results_into_a_local_file(
      be_verbose = true
    )
    unless @hash_available_programs.empty?
      what = YAML.dump(@hash_available_programs)
      # ===================================================================== #
      # The file will be stored under the temp_directory.
      # ===================================================================== #
      into = return_path_to_the_all_programs_file
      e "Storing the yaml-file into: #{sfile(into)}" if be_verbose
      write_what_into(what, into)
    end
  end; self.instance_eval { alias store_relevant_files consider_storing_these_results_into_a_local_file } # === EnvironmentInformation.store_relevant_files

  # ========================================================================= #
  # === Environment.silent_run
  # ========================================================================= #
  def self.silent_run
    @hash_available_programs = {} # Clear it every time this method is run.
    work_on_these_programs = []
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_NON_PROGRAMS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_PROGRAMS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_XORG_COMPONENTS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_SCIENCE_CLUSTER)
    work_on_these_programs.flatten.each {|this_program|
      version_of_this_program = ::EnvironmentInformation.return_version_of_this_program(this_program)
      if version_of_this_program
        version_of_this_program = version_of_this_program.to_s
      end
      @hash_available_programs[this_program] = version_of_this_program
    }
    consider_storing_these_results_into_a_local_file(false)
    return @hash_available_programs
  end; self.instance_eval { alias build_up_and_return_the_main_hash silent_run } # === EnvironmentInformation.build_up_and_return_the_main_hash
       self.instance_eval { alias initialize_hash                   silent_run } # === EnvironmentInformation.initialize_hash
       self.instance_eval { alias initialize                        silent_run } # === EnvironmentInformation.initialize
       self.instance_eval { alias run                               silent_run } # === EnvironmentInformation.run

  # ========================================================================= #
  # === EnvironmentInformation.show_detailed_information_about_all_available_components
  #
  # This method is mostly for testing purposes only; for real use
  # for the project you should consider making use of
  # Environment.silent_run instead.
  #
  # Do note that this method will always show all available components;
  # if you need more fine-tuned control then you have to use the
  # class EnvironmentInformation instead.
  # ========================================================================= #
  def self.show_detailed_information_about_all_available_components
    @hash_available_programs = {} # Clear it every time this method is run. 
    work_on_these_programs = []
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_NON_PROGRAMS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_PROGRAMS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_TRACKED_XORG_COMPONENTS)
    work_on_these_programs << YAML.load_file(FILE_ARRAY_SCIENCE_CLUSTER)
    work_on_these_programs.flatten!
    work_on_these_programs.uniq!
    e
    work_on_these_programs.each {|this_program|
      version_of_this_program = ::EnvironmentInformation.return_version_of_this_program(this_program)
      # ===================================================================== #
      # Do a little sanitizing next:
      # ===================================================================== #
      case this_program.to_s
      when 'kde_frameworks'
        this_program = 'KDE Frameworks'.to_sym
      end
      left_side = colourize_and_pad_the_left_side(this_program)
      case version_of_this_program
      when COMMAND_NOT_FOUND,
           ''
        version_of_this_program = nil
        colour_for_the_right_side = :orange
      else
        colour_for_the_right_side = :notify_if_missing
      end
      # ===================================================================== #
      # Next register this program into the main hash.
      # ===================================================================== #
      @hash_available_programs[this_program] = version_of_this_program
      right_side = colourize_and_pad_the_right_side(
        version_of_this_program,
        colour_for_the_right_side
      )
      e "  #{left_side} #{right_side}"
    }
    e
    # ======================================================================= #
    # Report missing programs.
    # ======================================================================= #
    _ = missing_components?
    unless _.empty?
      e 'The following components were not found:'
      e
      _.each {|this_component|
        e lightseagreen("  #{this_component}")
      }
      e
    end
    consider_storing_these_results_into_a_local_file
  end; self.instance_eval { alias test show_detailed_information_about_all_available_components } # === EnvironmentInformation.test

end

if __FILE__ == $PROGRAM_NAME
  EnvironmentInformation.test  
end