#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/prefix_to_use.rb'
# =========================================================================== #
module EnvironmentInformation

  # ========================================================================= #
  # === @prefix_to_use
  #
  # This variable can be used to designate another base directory to
  # be used, such as at /home/Programs/. This allows us to scan through
  # an AppDir and output the programs found there.
  #
  # By default it must be an empty String, as many other files check
  # on the content of this variable - so only change it if you really
  # need to use another prefix to use.
  #
  # The functionality was specifically added to support versioned
  # AppDirs.
  # ========================================================================= #
  @prefix_to_use = ''.dup

  # ========================================================================= #
  # === EnvironmentInformation.set_prefix_to_use
  # ========================================================================= #
  def self.set_prefix_to_use(i)
    @prefix_to_use = i.to_s.dup
  end; self.instance_eval { alias set_prefix set_prefix_to_use } # === EnvironmentInformation.set_prefix

  # ========================================================================= #
  # === EnvironmentInformation.prefix?
  # ========================================================================= #
  def self.prefix?
    @prefix_to_use
  end

end