#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/replay_from_the_stored_file.rb'
# EnvironmentInformation.colourize_and_pad_the_left_side
# =========================================================================== #
require 'yaml'
require 'environment_information/constants/temp_directory.rb'
require 'environment_information/colours/colours.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.colourize_and_pad_the_right_side
  # ========================================================================= #
  def self.colourize_and_pad_the_right_side(
      i,
      colour_for_the_right_side = ::EnvironmentInformation.colour_for_the_right_side
    )
    case colour_for_the_right_side
    # ======================================================================= #
    # === :notify_if_missing
    # ======================================================================= #
    when :notify_if_missing
      colour_for_the_right_side = ::EnvironmentInformation.colour_for_the_right_side
      if i.to_s.empty?
        i = '[Not found]'
        colour_for_the_right_side = :orange
      end
    end
    if use_colours?
      Colours.send(colour_for_the_right_side, i)
    else
      i
    end
  end; self.instance_eval { alias prepare_right_side colourize_and_pad_the_right_side } # === EnvironmentInformation.prepare_right_side

  # ========================================================================= #
  # === EnvironmentInformation.colourize_and_pad_the_left_side
  # ========================================================================= #
  def self.colourize_and_pad_the_left_side(i)
    left_side = i.to_s.ljust(32) # Pad to the left-hand side.
    if use_colours?
      "  #{Colours.send(::EnvironmentInformation.colour_for_the_left_side,  left_side)}"
    else
      "  #{left_side}"
    end
  end; self.instance_eval { alias prepare_left_side colourize_and_pad_the_left_side } # === EnvironmentInformation.prepare_left_side

  # ========================================================================= #
  # === EnvironmentInformation.replay_from_the_stored_file
  #
  # To invoke this method from the commandline, try:
  #
  #   envi --replay
  #
  # ========================================================================= #
  def self.replay_from_the_stored_file
    _ = return_path_to_the_all_programs_file
    if File.exist? _
      e "Loading from the file #{_}."
      @hash_available_programs = YAML.load_file(_)
      @hash_available_programs.each_pair {|key, value|
        
        e colourize_and_pad_the_left_side("#{key}:")+
          colourize_and_pad_the_right_side(
            value.to_s, :notify_if_missing
          )
      }
    else
      e "No file exists at #{_}."
    end
  end

  # ========================================================================= #
  # === EnvironmentInformation.return_path_to_the_all_programs_file
  # ========================================================================= #
  def self.return_path_to_the_all_programs_file
    "#{temp_directory?}all_programs.yml"
  end

end