#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# require 'environment_information/toplevel_methods/return_remote_gtk2_version.rb'
# EnvironmentInformation.return_remote_gtk2_version
# =========================================================================== #
require 'environment_information/toplevel_methods/internet_is_available.rb'

module EnvironmentInformation

  # ========================================================================= #
  # === EnvironmentInformation.return_remote_gtk2_version
  #
  # This method can be used to obtain the latest gtk2-version, from a
  # remote URL.
  #
  # The reason why this was necessary is because the RBT project may not
  # always keep the latest gtk2 version, since gtk3 (and so forth) is more
  # recent. Since we may still have to find out which gtk2 version is the
  # most recent, we need a method to do so - which is precisely what this
  # method here is doing.
  # ========================================================================= #
  def self.return_remote_gtk2_version
    # ======================================================================= #
    # We will use a hardcoded URL:
    # ======================================================================= #
    remote_url = 'https://ftp.gnome.org/pub/GNOME/sources/gtk+/2.24/?C=M;O=D'
    require 'open-uri'
    newest_version = ''
    # ======================================================================= #
    # We will next try to obtain the remote dataset, but this would
    # fail if we have no www-connection, so we must rescue this step.
    # ======================================================================= #
    if internet_is_available?
      begin
        dataset = URI.open(remote_url).read
        use_this_regex =
          /<a href="gtk\+\-(\d.\d\d.\d\d).tar.xz"><img src=/
        scanned = dataset.scan(use_this_regex).flatten
        newest_version = scanned.first
      rescue SocketError => error
        puts "It seems as if we have no working internet "\
             "connection (#{sfancy(error.class)})"
      end
    end
    return newest_version.strip # ← And return it here.
  end

end

if __FILE__ == $PROGRAM_NAME
  puts EnvironmentInformation.return_remote_gtk2_version
end